@@ -58,6 +58,11 @@ struct net_packet
       memmove ((void *)&(*this)[0], (void *)&(*this)[hdrsize], len);
     }
 
+  u16 ipv4_hdr_len () const
+    {
+      return ((*this)[0] & 15) << 2;
+    }
+
   void set (const net_packet &pkt)
     {
       len = pkt.len;
