@@ -2,7 +2,7 @@
     util.h -- process management and other utility functions
     Copyright (C) 1998-2002 Ivo Timmermans <ivo@o2w.nl>
                   2000-2002 Guus Sliepen <guus@sliepen.eu.org>
-                  2003-2011 Marc Lehmann <gvpe@schmorp.de>
+                  2003-2013 Marc Lehmann <gvpe@schmorp.de>
  
     This file is part of GVPE.
 
@@ -37,13 +37,12 @@
 #include <cstring>
 #include <sys/types.h>
 
-#include <openssl/rsa.h>
-
 #include "gettext.h"
 
 #include "slog.h"
 #include "ev_cpp.h"
 #include "callback.h"
+#include "global.h"
 
 typedef ev_tstamp tstamp;
 
@@ -126,30 +125,23 @@ typedef callback<const char *()> run_script_cb;
 // run a shell script (or actually an external program).
 pid_t run_script (const run_script_cb &cb, bool wait);
 
+void hexdump (const char *header, void *data, int len);
+
 #if ENABLE_HTTP_PROXY
 u8 *base64_encode (const u8 *data, unsigned int len);
 #endif
 
+/* always take more or less the same time to compare */
+bool slow_memeq (const void *a, const void *b, int len);
+
 /*****************************************************************************/
 
-typedef u8 rsaclear[RSA_KEYLEN - RSA_OVERHEAD]; // challenge data;
-typedef u8 rsacrypt[RSA_KEYLEN]; // encrypted challenge
+void rand_fill (void *data, int len);
 
-static inline void
-rsa_encrypt (RSA *key, const rsaclear &chg, rsacrypt &encr)
+template<class T>
+inline void rand_fill (T &t)
 {
-  if (RSA_public_encrypt (sizeof chg,
-                          (unsigned char *)&chg, (unsigned char *)&encr,
-                          key, RSA_PKCS1_OAEP_PADDING) < 0)
-    fatal ("RSA_public_encrypt error");
-}
-
-static inline bool
-rsa_decrypt (RSA *key, const rsacrypt &encr, rsaclear &chg)
-{
-  return RSA_private_decrypt (sizeof encr,
-                              (unsigned char *)&encr, (unsigned char *)&chg,
-                              key, RSA_PKCS1_OAEP_PADDING) > 0;
+  rand_fill (&t, sizeof (T));
 }
 
 /*****************************************************************************/
